#include "player.h"
#include "awoorwae.h"
#include <iostream>

namespace pabloader {
Player::Player(Awoorwae* game_)
    : Entity(game_)
{
}

olc::rcode Player::Load()
{
    const std::string spriteFile = "res/player.pgs";
    const std::string fallFile = "res/fall.wav";
    if (img.LoadFromPGESprFile(spriteFile, &game->pack)) {
        w = img.width / 4;
        h = img.height / 4;
        x = (game->ScreenWidth() - w) / 2;
        y = game->ScreenHeight() / 2;
    } else {
        std::cerr << "File " << spriteFile << " is not found" << std::endl;
        return olc::FAIL;
    }
    fallSample = olc::SOUND::LoadAudioSample(fallFile, &game->pack);
    if (fallSample < 0) {
        std::cerr << "File " << fallFile << " is not found" << std::endl;
        return olc::FAIL;
    }

    return olc::OK;
}

void Player::Update(float dt)
{
    x += xv * dt;
    y += yv * dt;

    xv *= 1 - 0.2 * dt;

    if (!onGround) {
        yv += 50 * dt;
        yv *= 1 - 0.05 * dt;
    } else {
        xv *= 1 - 0.5 * dt;
    }

    transform.Reset();

    game->DrawString(1, 1, "A,D - move; SPACE - jump", olc::CYAN);

#ifdef _DEBUG
    game->DrawString(1, 10, std::to_string(x) + " " + std::to_string(y));
    game->DrawString(1, 18, std::to_string(xv) + " " + std::to_string(yv));
#endif
    bool userInput = false;

    if (game->GetKey(olc::SPACE).bHeld && onGround) {
        yv = -100;
        userInput = true;
    }

    if (game->GetKey(olc::A).bHeld) {
        xv -= 100 * dt;
        userInput = true;
    }

    if (game->GetKey(olc::D).bHeld) {
        xv += 100 * dt;
        userInput = true;
    }

    if (game->GetKey(olc::J).bPressed) {
        skin = 1 - skin;
    }

    if (x > game->ScreenWidth() - w / 2) {
        xv *= -0.3;
        x = game->ScreenWidth() - w / 2;
        olc::SOUND::PlaySample(fallSample);
    }

    if (x < w / 2) {
        xv *= -0.3;
        x = w / 2;
        olc::SOUND::PlaySample(fallSample);
    }

    if (y > game->ScreenHeight() - h / 2) {
        yv *= -0.3;

        if (std::abs(yv) < 2) {
            yv = 0;
        }
        y = game->ScreenHeight() - h / 2;
    }

    bool newOnGround = y >= game->ScreenHeight() - h / 2;
    if (newOnGround && !onGround) {
        olc::SOUND::PlaySample(fallSample);
    }
    onGround = newOnGround;

    if (onGround) {
        tile = 2;
        tileCounter = 30;
    } else if (tileCounter <= 0) {
        tile = 0;
    } else {
        tileCounter--;
    }

    if (!userInput && std::abs(xv) < 2) {
        xv = 0;
    }
    if (xv < 0) {
        transform.Scale(-1, 1);
    }
    transform.Translate(x, y);
}

void Player::Draw()
{
    int xOff = xv < 0 ? -1 : 0;
    if (std::abs(xv) < 1) {
        olc::GFX2D::DrawPartialSprite(&img, skin * w + xOff, h + tile * h, w, h, transform, true);
    } else {
        olc::GFX2D::DrawPartialSprite(&img, skin * w + xOff, 0 + tile * h, w, h, transform, true);
    }
#ifdef _DEBUG
    float x, y;
    transform.Forward(0, 0, x, y);
    game->Draw((int)x, (int)y, olc::RED);
#endif
}
}